/*
 *  TT-ICS  Part of TTCHESS that connects to a mysqlserver
 *          and plays the moves on a Internet Chess Server
 * 
 *  Copyright (C) 200 J. Forsslund (jofo02@kth.se)
 *  All rights reserved.
 *
 *  TT-ICS is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  TT-ICS is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with TT-ICS; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

import java.util.Date;

import ictk.boardgame.chess.net.ics.ui.cli.ANSIConsole;
import ictk.boardgame.chess.net.ics.ICSProtocolHandler;
import ictk.boardgame.chess.net.ics.ICSEventRouter;
import ictk.boardgame.chess.net.ics.fics.FICSProtocolHandler;
import ictk.boardgame.chess.net.ics.event.ICSEvent;
import ictk.boardgame.chess.net.ics.event.ICSConnectionListener;
import ictk.boardgame.chess.net.ics.event.ICSConnectionEvent;

import java.sql.*;

public class ttics implements ICSConnectionListener, Runnable {
   String handle;
   String passwd;

    /** ttchess stuff **/
    moveList list;
    String fen;
    String moves;
    String white="0";
    String black="0";

    /** Game to play! **/
    String gid="49";


    String mysql_server="tmc3.hs.se";
    String mysql_user="ttview";
    String mysql_pass="******";
    String mysql_dbase="elitserien";


   ICSProtocolHandler ics;
   ANSIConsole ansiConsole;
    //   CommandInput commandLine;
    Thread t;
   
   /** creates a new SimpleICSClient registereing the ICS protocol handler
    *  and the default listener for all events.
    */
   public ttics(String handle, String passwd) {
      this.handle = handle;
      this.passwd = passwd;

      ics = new FICSProtocolHandler();
      ics.addConnectionListener(this);

      ansiConsole = new ANSIConsole();

      ICSEventRouter router = ics.getEventRouter();
      router.setDefaultListener(ansiConsole);

      //channel 1 (help) goes to a different listener
      //      router.addChannelListener(ICSEvent.CHANNEL_EVENT, 1,
      //				new ChannelListenerExample());

      //need all other channel events to still go to ansiConsole
      ics.getEventRouter().addEventListener(ICSEvent.CHANNEL_EVENT, ansiConsole);

      //so the main ChannelEvent listener doesn't also get the event
      router.setChannelExclusive(ICSEvent.CHANNEL_EVENT, 1, true);
      //so the defaultListener doesn't get the event
      router.setExclusive(ICSEvent.CHANNEL_EVENT, true);

      //      commandLine = new CommandInput("Simple ICS Client", ics);

      mysql_connect();
      list = new moveList();
      mysql_update();
   }


    /** Mysql functions **/
    void mysql_update(){
	list.clear();
      	mysql_execute();

		white=getPlayer(white);
		black=getPlayer(black);

	list.set(moves);
	//	list.add("(Current)",board);
    }


    
    Connection conn = null;
    Statement stmt = null;


    String mysql_connect(){
		try
		{
			Class.forName("com.mysql.jdbc.Driver").newInstance();
			conn = DriverManager.getConnection("jdbc:mysql://"+mysql_server+"/"+mysql_dbase+"?user="+mysql_user+"&password="+mysql_pass);
			//message="Connected to database";

			return "";
		}
		catch (SQLException e)
		{
			return e.getSQLState();
		}
		catch (Exception e)
		{
			return "0";
		}
	}

	boolean mysql_disconnect()
	{
		if (conn != null) {
			try
			{
				conn.close();
				return true;
			}
			catch (SQLException SQLE)
			{
				return false;
			}
		}
		else
			return false;
	}



	String mysql_execute()
	{
		try
		{
			stmt = null;
			stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT white, black, moves, fen from games where id="+gid);
			while (rs.next()) {
			    fen = rs.getString("fen");
			    white = rs.getString("white");
			    black = rs.getString("black");
			    moves = rs.getString("moves");
			    //			    board=fen2board(fen);
			}

			return "";
		}
		catch (SQLException e)
		{
			return e.getSQLState();
		}
		catch (Exception e)
		{
			return "0";
		}
		finally
		{

		    
			if (stmt != null) {
				try
				{
					stmt.close();
				}
				catch (SQLException SQLE)
				{
					;
				}
			}
		    
		}
	}

	String getPlayer(String id)
	{
	    String r="";
		try
		{
			stmt = null;
			stmt = conn.createStatement();
			ResultSet rs = stmt.executeQuery("SELECT titel, firstname, lastname from players where id="+id);
			while (rs.next()) {
			    
			    String t=(rs.getString("titel")).trim();
			    if(t.length()>0) r+=t + "_";

			    String lname=(rs.getString("lastname")).trim();

			    if(lname.indexOf(" ")>0)
				lname=lname.substring(0,lname.indexOf(" "));

			    r+=lname+",";

			    String fname=(rs.getString("firstname")).trim();
			    if(fname.length()>0)
				r+=fname.substring(0,1);

			    if(r.length()>16)
				r=r.substring(0,16);
			}

			return r;
		}
		catch (SQLException e)
		{
			return e.getSQLState();
		}
		catch (Exception e)
		{
			return "0";
		}
		finally
		{

		    
			if (stmt != null) {
				try
				{
					stmt.close();
				}
				catch (SQLException SQLE)
				{
					;
				}
			}
		    
		}
	}





    long starttime=0;
    int counter=0;

    public void run(){
	while(Thread.currentThread()==t){

	    //	    message= "Time: "+System.currentTimeMillis();

	    try{
		if(starttime==0) starttime=System.currentTimeMillis();

		Thread.sleep(2000);

		/** Because ics.isConnected() wont work for any reason **/
		if(System.currentTimeMillis() - starttime > 15000){
		    java.util.Date d=new java.util.Date();
		    String message="Last update:"+d.toString();

		    System.out.println(message);

		    if(counter==0){
			ics.sendCommand("examine");
		Thread.sleep(500);
			ics.sendCommand("wname "+white);
		Thread.sleep(500);
			ics.sendCommand("bname "+black);
		Thread.sleep(500);
			ics.sendCommand("wclock 0:42:48");
		Thread.sleep(500);
			ics.sendCommand("bclock 1:03:17");
		Thread.sleep(500);
			ics.sendCommand("unpause");
		    }

		    
		    if(counter>0){
			if(counter<=list.getHalfmoves())
			    ics.sendCommand(list.getMove(counter));
		    }
		    
		    /** Tested; we may make all moves at once!
		    if(counter>0){
			for(int i=1;i<=list.getHalfmoves();i++)
			    ics.sendCommand(list.getMove(i));
		    }
		    */

		    counter++;
		    
		    /** Tested; a static game
		    String cmd="";

		    switch(counter){
		    case 0: cmd="examine"; break;
		    case 1: cmd="d4"; break;
		    case 2: cmd="Nf6"; break;
		    case 3: cmd="c4"; break;
		    case 4: cmd="c6"; break;
		    case 5: cmd="Nf3"; break;
		    case 6: cmd="d5"; break;
		    case 7: cmd="Nc3"; break;
		    case 8: cmd="a6"; break;
		    case 9: cmd="c5"; break;
		    case 10: cmd="Nb8d7"; break;
		    case 11: cmd="h3"; break;
		    case 12: cmd="Qc7"; break;
		    case 13: cmd="Bg5"; break;
		    case 14: cmd="h6"; break;
		    case 15: cmd="Bh4"; break;
		    case 16: cmd="Nh5"; break;
		    case 17: cmd="g4"; break;
		    case 18: cmd="g5"; break;
		    case 19: cmd="gxh5"; break;
		    case 20: cmd="gxh4"; break;
		    case 21: cmd="e3"; break;
		    case 22: cmd="b6"; break;
		    case 23: cmd="cxb6"; break;
		    case 24: cmd="Qxb6"; break;
		    case 25: cmd="Qc2"; break;
		    case 26: cmd="e6"; break;
		    case 27: cmd="O-O-O"; break;
		    case 28: cmd="c5"; break;
		    case 29: cmd="Rg1"; break;
		    case 30: cmd="Rb8"; break;

		    }

		    if(!cmd.equals(""))
			ics.sendCommand(cmd);

		    counter++;

		    */
		}


	    } catch (InterruptedException e) {
		break;
	    }
	}

    }


    /** if not already connected this will establish a connection to the
     *  server.
     */
    public void connect() {
       // Add your handling code here:
       if (!ics.isConnected()) {
          try {
	     System.out.println("[Client] attempting to connect");
	     ics.setHandle(handle);
	     ics.setPassword(passwd);
	     ics.setLagCompensation(true);
             ics.connect();

	     t=new Thread(this);
	     t.start();

	  }
	  catch (java.net.UnknownHostException e) {
	     e.printStackTrace();
	  }
	  catch (java.io.IOException e) {
	     e.printStackTrace();
	  }
       }
    }
    /*
    public void setVisible (boolean t) {
       commandLine.setVisible(t);
    }
    */
    public void connectionStatusChanged (ICSConnectionEvent evt) {
       ICSProtocolHandler conn = evt.getConnection();

       if (!conn.isConnected()) {
          System.err.println("Connection Terminated: " + new Date());
          System.exit(0);
       }
       else {
          System.err.println("Connection Live but received event: " + evt);
       }
    }

    /**
    * @param args the command line arguments
    */
    public static void main(String args[]) {
       ttics client = null;
       if (args.length < 2) {
          System.err.println("You must supply username and password");
	  System.exit(1);
       }
       client = new ttics(args[0], args[1]);
       //       client.setVisible(true);
       client.connect();
    }
}
